package cn.com.qmth.examcloud.core.basic.starter;

import org.apache.logging.log4j.ThreadContext;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.autoconfigure.domain.EntityScan;
import org.springframework.cloud.client.discovery.EnableDiscoveryClient;
import org.springframework.cloud.netflix.eureka.EnableEurekaClient;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.jpa.repository.config.EnableJpaAuditing;
import org.springframework.data.jpa.repository.config.EnableJpaRepositories;
import org.springframework.transaction.annotation.EnableTransactionManagement;

import cn.com.qmth.examcloud.core.basic.service.cache.AppCache;
import cn.com.qmth.examcloud.web.bootstrap.AppBootstrap;
import cn.com.qmth.examcloud.web.support.SpringContextHolder;

@SpringBootApplication
@Configuration
@EnableAutoConfiguration
@EnableJpaAuditing
@EnableTransactionManagement
@EnableEurekaClient
@EnableDiscoveryClient
@ComponentScan(basePackages = {"cn.com.qmth"})
@EntityScan(basePackages = {"cn.com.qmth"})
@EnableJpaRepositories(basePackages = {"cn.com.qmth"})
public class CoreBasicApp {

	static {
		String runtimeLevel = System.getProperty("log.commonLevel");
		if (null == runtimeLevel) {
			System.setProperty("log.commonLevel", "DEBUG");
		}
		System.setProperty("hibernate.dialect.storage_engine", "innodb");
	}

	/**
	 * main
	 *
	 * @author WANGWEI
	 * @param args
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {

		AppBootstrap.run(CoreBasicApp.class, args);

		initCache();
	}

	/**
	 * 初始化缓存
	 *
	 * @author WANGWEI
	 */
	private static void initCache() {
		new Thread(new Runnable() {
			@Override
			public void run() {
				ThreadContext.put("TRACE_ID", Thread.currentThread().getName());
				SpringContextHolder.getBean(AppCache.class).loadAllFromResource();
			}
		}, "cache").start();
	}
}
